==> silk-2.3.1.tar.gz: silk-2.3.1/src/rwuniq/rwuniq.h <==
/*
** Copyright (C) 2001-2010 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _RWUNIQ_H
#define _RWUNIQ_H

/*
**  rwuniq.h
**
**  Common declarations for the rwuniq application.  See rwuniq.c for
**  an explanation.
*/


#include <silk/silk.h>

RCSIDENTVAR(rcsID_RWUNIQ_H, "$SiLK: rwuniq.h 15604 2010-04-08 19:01:04Z mthomas $");

#include <silk/utils.h>
#include <silk/rwascii.h>
#include <silk/skstream.h>
#include <silk/iochecks.h>
#include <silk/hashlib.h>
#include <silk/skvector.h>
#include <silk/iptree.h>
#include <silk/skplugin.h>


/* TYPEDEFS AND DEFINES */

/* For the compound key version of rwuniq, the maximum number of bytes
 * we allow in the key.  An arbitrary number that should be big enough
 * for almost all cases. */
#define MAX_KEY_BYTE_WIDTH 256

/* For ASCII output, maximum size of a single value column. Must be
 * big enough to hold printed uint64_t, sktimestamp(), class, type,
 * sensor, or arbitrary pmap value. */
#define VALUE_OUT_BUF_SIZE  256

/* Max size of a value in the hash table.  Must be able to hold the
 * values: bytes(8), packets(4), flows(4), sTime(4), and eTime(4), and
 * any plugin values.  */
#define MAX_VALUE_BYTE_WIDTH 256

/* Max size of a key/value pair in the hash table.  Must be able to
 * hold the max_key, distinct sIP(4) & dIP(4), and the max_value byte
 * width. */
#define MAX_HASH_NODE_BYTES (MAX_KEY_BYTE_WIDTH + 8 + MAX_VALUE_BYTE_WIDTH)

/* Value to use as exit code when we run out of memory */
#define RWUNIQ_NO_MEMORY_EXIT_CODE 16

/* default sTime bin size to use when --bin-time is requested */
#define DEFAULT_TIME_BIN  60

/* whether we need to use memcpy() or can access unaligned memory */
#ifndef RWUNIQ_USE_MEMCPY
#ifdef HAVE_ALIGNED_ACCESS_REQUIRED
#define RWUNIQ_USE_MEMCPY 1
#else
#define RWUNIQ_USE_MEMCPY 0
#endif
#endif

/* function that gets the local part of the key from an rwrec */
typedef void (*rec2key_fn_t)(const rwRec *rec, void *key_pos);

/* function that puts the key into the rwrec */
typedef void (*key2rec_fn_t)(rwRec *rec, const void *key_pos);

/* struct to hold the columns/fields that make up our key for this run */
typedef struct key_field_st {
    /* a buffer to hold the textual value for this field when printing
     * the results */
    char              kf_textbuf[VALUE_OUT_BUF_SIZE];
    /* the functions to call to fill in this field.  May be NULL.
     * When these are set, they are used to convert a member of an
     * rwRec into a binary value, and to convert the binary value back
     * to members of the rwRec. */
    rec2key_fn_t      kf_r2k;
    key2rec_fn_t      kf_k2r;
    /* If non-null, the plugin field handle to use for this field */
    skplugin_field_t *kf_field_handle;
    /* the ID of the column, from parseFields(), that makes up this
     * key_field */
    uint32_t          kf_id;
    /* the byte-offset where this field begins in the binary key used
     * for binning. */
    size_t            kf_offset;
    /* the byte-width of this field in the binary key */
    size_t            kf_width;
} key_field_t;


/*
 *  The built-in value fields to count w.r.t. the key.  Keep this in
 *  sync with the potential_values[] array in rwuniqsetup.c, and with
 *  the OPT_COUNT_* values in appOptionsEnum.
 */
typedef enum {
    VALUE_BYTES = 0,
    VALUE_PACKETS,
    VALUE_FLOWS,
    VALUE_STIME,
    VALUE_ETIME,
    DISTINCT_SIP,
    DISTINCT_DIP,
    VALUE_PLUGIN
} value_types_t;


/* struct to hold information about the value fields */
typedef struct value_field_st {
    /* the title of this field */
    const char       *vf_title;
    /* the plugin information for this field (if any) */
    skplugin_field_t *vf_plugin_field;
    /* only print sums if the minimum value is at least this value */
    uint64_t          vf_min;
    /* only print sums if the maximum value is no more than this value */
    uint64_t          vf_max;
    /* the byte-width of this field */
    size_t            vf_byte_width;
    /* the byte-offset for each value field */
    size_t            vf_offset;
    /* the text width of the field for columnar output */
    int               vf_text_len;
    /* the id for this column */
    value_types_t     vf_id;
} value_field_t;


/* The number of distinct fields that are supported_ */
#define NUM_DISTINCTS 2

/* struct to hold information about distinct fields */
typedef struct distinct_field_st {
    /* function to call to pack this field into key */
    rec2key_fn_t        df_r2k;
    /* the byte-width of this field when used in th key */
    size_t              df_byte_width;
    /* the byte-offset for this field in the key */
    size_t              df_offset;
    /* the text width of the field for columnar output */
    int                 df_text_len;
    /* the id for this column */
    value_types_t       df_id;
    /* the location of this field in the value_fields[] */
    int                 df_val_field_pos;
} distinct_field_t;

/* group input/output variables into a single struct */
typedef struct output_st {
    FILE                 *fp;
    const char           *pathname;
    int                   ispipe;
} app_output_t;

/* flags set by user options */
typedef struct app_flags_st {
    unsigned presorted_input    :1;      /* Assume input is sorted */
    unsigned sort_output        :1;      /* Whether to sort the output */
    unsigned print_filenames    :1;
    unsigned no_columns         :1;
    unsigned no_titles          :1;
    unsigned no_final_delimiter :1;
    unsigned integer_sensors    :1;
    unsigned integer_ips        :1;
    unsigned zero_pad_ips       :1;
} app_flags_t;


/* VARIABLE DECLARATIONSS */

/* delimiter between output columns */
extern char delimiter;

/* the columns/fields that make up our key for this run */
extern key_field_t *key_fields;

/* the number of actual columns/fields that make up the key */
extern size_t key_num_fields;

/* the total byte count of the key */
extern size_t key_byte_width;

/* The actual value fields that we are using */
extern value_field_t *value_fields;

/* number of value fields the user has requested */
extern size_t value_num_fields;

/* total byte count for all value_fields that are being computed */
extern size_t value_byte_width;

/* the initial values as a byte-encoded array */
extern uint8_t *value_initial_vals;

/* Information about each distinct field. */
extern distinct_field_t distinct_fields[NUM_DISTINCTS];

/* number of distinct fields the user has requested */
extern size_t distinct_num_fields;

/* the byte length of the key used by the hash table, which is the
 * length of the "key" and any "distinct" fields  */
extern size_t hash_key_bytes;

/* the byte length of the value used by the hash table, which has a
 * minimum length of 1  */
extern size_t hash_value_bytes;

/* the byte length of the key-value pair in the hash table, which is
 * comprised of the key and the value */
extern size_t hash_node_bytes;

/* to convert the key fields (as an rwRec) to ascii */
extern rwAsciiStream_t *ascii_str;

/* the real output */
extern app_output_t output;

/* flags set by the user options */
extern app_flags_t app_flags;

/* how to print timestamps */
extern uint32_t time_flags;

/* how to handle IPv6 flows */
extern sk_ipv6policy_t ipv6_policy;


/* FUNCTION DECLARATIONS */

void appSetup(int argc, char **argv);
void appTeardown(void);
void appExit(int status);
int  appNextInput(skstream_t **rwios);

#endif /* _RWUNIQ_H */

/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/

==> silk-2.4.5.tar.gz: silk-2.4.5/src/rwuniq/rwuniq.h <==
/*
** Copyright (C) 2001-2011 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract FA8721-05-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/
#ifndef _RWUNIQ_H
#define _RWUNIQ_H

/*
**  rwuniq.h
**
**  Common declarations for the rwuniq application.  See rwuniq.c for
**  an explanation.
*/


#include <silk/silk.h>

RCSIDENTVAR(rcsID_RWUNIQ_H, "$SiLK: rwuniq.h 16700 2011-02-10 18:34:03Z mthomas $");

#include <silk/utils.h>
#include <silk/rwascii.h>
#include <silk/skstream.h>
#include <silk/hashlib.h>
#include <silk/skvector.h>
#include <silk/iptree.h>
#include <silk/skplugin.h>
#include <silk/sktempfile.h>

/* use TRACEMSG_LEVEL as our tracing variable */
#define TRACEMSG(msg) TRACEMSG_TO_TRACEMSGLVL(1, msg)
#include <silk/sktracemsg.h>


/* TYPEDEFS AND DEFINES */

/* For the compound key version of rwuniq, the maximum number of bytes
 * we allow in the key.  An arbitrary number that should be big enough
 * for almost all cases. */
#define MAX_KEY_BYTE_WIDTH 256

/* For ASCII output, maximum size of a single value column. Must be
 * big enough to hold printed uint64_t, sktimestamp(), class, type,
 * sensor, or arbitrary pmap value. */
#define VALUE_OUT_BUF_SIZE  256

/* Max size of a value in the hash table.  Must be able to hold the
 * values: bytes(8), packets(4), flows(4), sTime(4), and eTime(4), and
 * any plugin values.  */
#define MAX_VALUE_BYTE_WIDTH 256

/* Max size of a key/value pair in the hash table.  Must be able to
 * hold the max_key, distinct sIP(4) & dIP(4), and the max_value byte
 * width. */
#define MAX_HASH_NODE_BYTES (MAX_KEY_BYTE_WIDTH + 8 + MAX_VALUE_BYTE_WIDTH)

/* Value to use as exit code when we run out of memory */
#define RWUNIQ_NO_MEMORY_EXIT_CODE 16

/* default sTime bin size to use when --bin-time is requested */
#define DEFAULT_TIME_BIN  60

/* whether we need to use memcpy() or can access unaligned memory */
#ifndef RWUNIQ_USE_MEMCPY
#ifdef SK_HAVE_ALIGNED_ACCESS_REQUIRED
#define RWUNIQ_USE_MEMCPY 1
#else
#define RWUNIQ_USE_MEMCPY 0
#endif
#endif

/* function that gets the local part of the key from an rwrec */
typedef void (*rec2key_fn_t)(const rwRec *rec, void *key_pos);

/* function that puts the key into the rwrec */
typedef void (*key2rec_fn_t)(rwRec *rec, const void *key_pos);

/* struct to hold the columns/fields that make up our key for this run */
typedef struct key_field_st {
    /* a buffer to hold the textual value for this field when printing
     * the results */
    char              kf_textbuf[VALUE_OUT_BUF_SIZE];
    /* the functions to call to fill in this field.  May be NULL.
     * When these are set, they are used to convert a member of an
     * rwRec into a binary value, and to convert the binary value back
     * to members of the rwRec. */
    rec2key_fn_t      kf_r2k;
    key2rec_fn_t      kf_k2r;
    /* If non-null, the plugin field handle to use for this field */
    skplugin_field_t *kf_field_handle;
    /* the ID of the column, from parseFields(), that makes up this
     * key_field */
    uint32_t          kf_id;
    /* the byte-offset where this field begins in the binary key used
     * for binning. */
    size_t            kf_offset;
    /* the byte-width of this field in the binary key */
    size_t            kf_width;
} key_field_t;


/*
 *  The built-in value fields to count w.r.t. the key.  Keep this in
 *  sync with the builtin_values[] array in rwuniqsetup.c, and with
 *  the OPT_COUNT_* values in appOptionsEnum.
 */
typedef enum {
    VALUE_BYTES = 0,
    VALUE_PACKETS,
    VALUE_FLOWS,
    VALUE_STIME,
    VALUE_ETIME,
    DISTINCT_SIP,
    DISTINCT_DIP,
    VALUE_PLUGIN
} value_types_t;


/* struct to hold information about the value fields */
typedef struct value_field_st {
    /* the title of this field */
    const char       *vf_title;
    /* the plugin information for this field (if any) */
    skplugin_field_t *vf_plugin_field;
    /* only print sums if the minimum value is at least this value */
    uint64_t          vf_min;
    /* only print sums if the maximum value is no more than this value */
    uint64_t          vf_max;
    /* the byte-width of this field */
    size_t            vf_byte_width;
    /* the byte-offset for each value field */
    size_t            vf_offset;
    /* the text width of the field for columnar output */
    int               vf_text_len;
    /* the id for this column */
    value_types_t     vf_id;
} value_field_t;


/* The number of distinct fields that are supported_ */
#define NUM_DISTINCTS 2

/* struct to hold information about distinct fields */
typedef struct distinct_field_st {
    /* function to call to pack this field into key */
    rec2key_fn_t        df_r2k;
    /* the byte-width of this field when used in th key */
    size_t              df_byte_width;
    /* the byte-offset for this field in the key */
    size_t              df_offset;
    /* the text width of the field for columnar output */
    int                 df_text_len;
    /* the id for this column */
    value_types_t       df_id;
    /* the location of this field in the value_fields[] */
    int                 df_val_field_pos;
} distinct_field_t;

/* group input/output variables into a single struct */
typedef struct output_st {
    FILE                 *fp;
    const char           *pathname;
    int                   ispipe;
} app_output_t;

/* flags set by user options */
typedef struct app_flags_st {
    unsigned presorted_input    :1;      /* Assume input is sorted */
    unsigned sort_output        :1;      /* Whether to sort the output */
    unsigned print_filenames    :1;
    unsigned no_columns         :1;
    unsigned no_titles          :1;
    unsigned no_final_delimiter :1;
    unsigned integer_sensors    :1;
    unsigned integer_tcp_flags  :1;
    unsigned integer_ips        :1;
    unsigned zero_pad_ips       :1;
} app_flags_t;


/* VARIABLE DECLARATIONS */

/* delimiter between output columns */
extern char delimiter;

/* the columns/fields that make up our key for this run */
extern key_field_t *key_fields;

/* the number of actual columns/fields that make up the key */
extern size_t key_num_fields;

/* the total byte count of the key */
extern size_t key_byte_width;

/* The actual value fields that we are using */
extern value_field_t *value_fields;

/* number of value fields the user has requested */
extern size_t value_num_fields;

/* total byte count for all value_fields that are being computed */
extern size_t value_byte_width;

/* the initial values as a byte-encoded array */
extern uint8_t *value_initial_vals;

/* Information about each distinct field. */
extern distinct_field_t distinct_fields[NUM_DISTINCTS];

/* number of distinct fields the user has requested */
extern size_t distinct_num_fields;

/* the byte length of the key used by the hash table, which is the
 * length of the "key" and any "distinct" fields  */
extern size_t hash_key_bytes;

/* the byte length of the value used by the hash table, which has a
 * minimum length of 1  */
extern size_t hash_value_bytes;

/* the byte length of the key-value pair in the hash table, which is
 * comprised of the key and the value */
extern size_t hash_node_bytes;

/* to convert the key fields (as an rwRec) to ascii */
extern rwAsciiStream_t *ascii_str;

/* the real output */
extern app_output_t output;

/* flags set by the user options */
extern app_flags_t app_flags;

/* how to print timestamps */
extern uint32_t time_flags;

/* how to handle IPv6 flows */
extern sk_ipv6policy_t ipv6_policy;


/* FUNCTION DECLARATIONS */

void appSetup(int argc, char **argv);
void appTeardown(void);
void appExit(int status);
int  appNextInput(skstream_t **rwios);

#endif /* _RWUNIQ_H */

/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
